#
# ATProject.py
#
# Proyecto de documentación, software, etc.
#
# Alejandro Álvarez Ayllón - 01/04/2006
#
#

from Products.Archetypes.public import BaseFolderSchema, Schema
from Products.Archetypes.public import StringField, TextField, ImageField, LinesField
from Products.Archetypes.public import ImageWidget, StringWidget, TextAreaWidget, LinesWidget
from Products.Archetypes.public import SelectionWidget, MultiSelectionWidget
from Products.Archetypes.public import registerType

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

from AccessControl import ClassSecurityInfo

import Utils

from config import PROJECTNAME, DEVELOPMENT_STATUS_LIST, LICENSE_LIST
from config import OPERATIVE_SYSTEM_LIST, PROGRAMMING_LANGUAGE_LIST
from config import TOPIC_LIST, TRANSLATION_LIST, I18N_DOMAIN

# Campos
schema = BaseFolderSchema + Schema((
    StringField('unix_name',
                searchable = 1,
                required   = 1,
                validators = ('isValidId'),
                mutator    = 'setUnixName',
                widget     = StringWidget(label       = 'Unix name',
                                          label_msgid = 'unix_name',
                                          i18n_domain = I18N_DOMAIN,
                                          ),
              ),
    TextField('long_description',
              searchable = 0,
              widget     = TextAreaWidget(label       = 'Full description',
                                          label_msgid = 'full_description',
                                          i18n_domain = I18N_DOMAIN,
                                          ),
              ),
    StringField('status',
                searchable = 1,
                widget     = SelectionWidget(label       = 'Development status',
                                             label_msgid = 'dev_status',
                                             i18n_domain = I18N_DOMAIN,
                                             ),
                vocabulary = DEVELOPMENT_STATUS_LIST,
                ),
    StringField('license',
                searchable = 1,
                widget     = SelectionWidget(label       = 'License',
                                             label_msgid = 'license',
                                             i18n_domain = I18N_DOMAIN,
                                             ),
                vocabulary = LICENSE_LIST,
                ),
    LinesField('operative_system',
                searchable = 1,
                widget     = MultiSelectionWidget(label       = 'Operative system',
                                                  label_msgid = 'operative_system',
                                                  i18n_domain = I18N_DOMAIN,
                                                  ),
                vocabulary = OPERATIVE_SYSTEM_LIST,
                ),
    LinesField('programming_language',
                searchable = 1,
                widget     = MultiSelectionWidget(label       = 'Programming language',
                                                  label_msgid = 'programming_language',
                                                  i18n_domain = I18N_DOMAIN,
                                                  ),
                vocabulary = PROGRAMMING_LANGUAGE_LIST,
                ),
    LinesField('topic',
                searchable = 1,
                widget     = MultiSelectionWidget(label       = 'Topic',
                                                  label_msgid = 'topic',
                                                  i18n_domain = I18N_DOMAIN,
                                                  ),
                vocabulary = TOPIC_LIST,
                ),
    LinesField('translation',
                searchable = 1,
                widget     = MultiSelectionWidget(label       = 'Translations',
                                                  label_msgid = 'translations',
                                                  i18n_domain = I18N_DOMAIN,
                                                  ),
                vocabulary = TRANSLATION_LIST,
                ),
    LinesField('administrators',
               searchable = 1,
               mutator    = 'setAdministrators',
               widget     = LinesWidget(label       = 'Administrators',
                                        label_msgid = 'administrators',
                                        i18n_domain = I18N_DOMAIN,
                                        ),
               ),
    LinesField('members',
               searchable = 1,
               mutator    = 'setMembers',
               widget     = LinesWidget(label       = 'Members',
                                        label_msgid = 'members',
                                        i18n_domain = I18N_DOMAIN,
                                        ),
               ),
    LinesField('mail_list',
               searchable = 0,
               mutator    = 'setPublicMailListMembers',
               widget     = LinesWidget(label       = 'Mail List',
                                        label_msgid = 'mail_list',
                                        i18n_domain = I18N_DOMAIN,
                                        ),
               accessor      = 'getPublicMailListMembers',
               edit_accessor = 'getPublicMailListMembers'
               ),
    ),)

# Clase
class ATProject(Utils.BaseFilteredFolder):
    """Project"""

    schema = schema

    content_icon    = 'project_icon.gif'
    meta_type       = 'ATProject'
    portal_type     = 'Project'
    archetype_name  = 'Project'
    typeDescription = 'Project'

    actions = ({
        'id'         : 'view',
        'name'       : 'View',
        'action'     : 'string:${object_url}/project_info',
        'permissions': (CMFCorePermissions.View,)
        },
               )

    security = ClassSecurityInfo()
 
    # Lista de tipos admitidos
    allowedTypes = ['News Item', 'Image', 'DocumentFolder', 'ProjectPackage', 'BugTracking']

    # Después de añadir
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """
        Descripción: Ajusta los permisos del propietario y los administradores al crearlo.
                     (Manejador de evento)
        Parámetros:  item y container, aunque no hace uso de ellos.
        Devuelve:    Nada
        """
        Utils.BaseFilteredFolder.manage_afterAdd(self, item, container)
        self.manage_permission(CMFCorePermissions.AddPortalContent,
                               roles   = ['Manager', 'Owner'],
                               acquire = 0
                               )
        
        # Añadimos un bugtracking si no existe ya
        if len(self.objectValues(['ATBugTracking',])) == 0:
            oid = 'BugTracking'
            self.invokeFactory('BugTracking', oid)
            obj = self._getOb(oid)
            obj.title = 'BugTracking'

    # Método para acceder al repositorio del proyecto
    security.declareProtected(CMFCorePermissions.View, 'viewcvs')
    def viewcvs(self):
        """
        Descripción: Accede al repositorio mediante forja_tool.
        Parámetros:  Ninguno
        Devuelve:    El código HTML devuelto por el método de forja_tool.
        """
        forja_tool = getToolByName(self, 'forja_tool')
        
        return forja_tool.ViewCVS(self.unix_name, self.getPhysicalPath())
        

    ### Métodos para manejar los usuarios ###
    
    security.declareProtected(CMFCorePermissions.View, 'IsAdmin')
    def IsAdmin(self, user):
        """
        Descripción: Indica si el usuario pasado es administrador del proyecto.
        Parámetros:  'user', el nombre del usuario que se quiere comprobar.
        Devuelve:    True si es administrador, False si no.
        """
        return user == self.Creator().encode() or \
               unicode(user, 'utf-8') in self.administators

    security.declareProtected(CMFCorePermissions.View, 'IsMember')
    def IsMember(self, user):
        """
        Descripción: Indica si el usuario pasado es miembro del proyecto.
        Parámetros:  'user', el nombre del usuario que se quiere comprobar.
        Devuelve:    True si es administrador, False si no.
        """
        return unicode(user, 'utf-8') in self.members

    # Devuelve el URL del repositorio
    security.declareProtected(CMFCorePermissions.View, 'getRepository')
    def getRepository(self):
        """
        Descripción: Devuelve la URL del repositorio.
        Parámetros:  Ninguno
        Devuelve:    La URL del repositorio.
        """
        forja_tool = getToolByName(self, 'forja_tool')

        return '%s/%s' % (forja_tool.repository_host, self.unix_name)


    # Esta función se emplea para modificar el nombre Unix
    # ya que, al hacerlo, es necesario cambiar el ID y modificar el
    # nombre del repositorio
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setUnixName')
    def setUnixName(self, value):
        """
        Descripción: Cambia el nombre UNIX del proyecto.
        Parámetros:  El nuevo nombre.
        Devuelve:    Nada
        """

        forja_tool   = getToolByName(self, 'forja_tool')
        membership   = getToolByName(self, 'portal_membership')

        # Si ha cambiado
        if self.unix_name != value:
            # Cambiamos (o creamos) el repositorio
            if forja_tool.enable_repository:
                forja_tool.moveOrCreateRepository(self.unix_name,
                                                  value,
                                                  self.Creator())

            # Cambiamos o creamos las listas de correo
            if forja_tool.enable_maillists:
                mailman_tool = getToolByName(self, 'mailman_tool')
                
                listAdmin       = membership.getMemberById(self.Creator()).getProperty('email')
                listAdminPasswd = membership.getUserPassword(self.Creator())

                # Lista para los usuarios
                if mailman_tool.listExists('%s-users' % self.unix_name):
                    mailman_tool.removeList('%s-users' % self.unix_name)
                    
                mailman_tool.newList('%s-users' % value,
                                     listAdmin, listAdminPasswd)

                # Lista para los desarrolladores
                if mailman_tool.listExists('%s-dev' % self.unix_name):
                    mailman_tool.removeList('%s-dev' % self.unix_name)

                mailman_tool.newList('%s-dev' % value,
                                     listAdmin, listAdminPasswd)

            # Cambiamos el ID
            self.unix_name = value
            self.setId(self.unix_name)

    # Cambiamos los administradores
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setAdministrators')
    def setAdministrators(self, list, update = True):
        """
        Descripción: Cambiar los administradores.
        Parámetros:  list   = Lista/vector/tupla de los nombres de los administradores.
                     update = Booleano que indica si actualizar o no el repositorio.
                              Por defecto es True.
        Devuelve:    Nada
        """

        forja_tool = getToolByName(self, 'forja_tool')
        membership = getToolByName(self, 'portal_membership')
        
        self.administrators = []

        # Eliminamos los administradores que no están ya
        for i in self.__ac_local_roles__.keys():
            if 'Manager'  in self.__ac_local_roles__[i] and not i.encode() in list and \
                   i != self.Creator():
                del self.__ac_local_roles__[i]

        # Establecemos los que están (pero no son miembros)
        for i in list:
            if membership.getMemberById(i) is None:
                raise 'The user %s does not exist' % i
            elif i != self.Creator() and i not in self.members:
                self.__ac_local_roles__[i.decode()] = ['Manager',]
                self.administrators = self.administrators + [i,]

        # Actualizamos el archivo passwd del repositorio
        if update and forja_tool.enable_repository:
            forja_tool.updateProjectRepositoryMembers(self.unix_name,
                                                      self.Creator(),
                                                      self.administrators,
                                                      self.members)
        # Actualizamos la lista de desarrolladores
        if update and forja_tool.enable_maillists:
            mailman_tool = getToolByName(self, 'mailman_tool')
            membership   = getToolByName(self, 'portal_membership')
            mails = []
            for i in self.administrators:
                mails.append(membership.getMemberById(i).getProperty('email'))
                
            for i in self.members:
                mails.append(membership.getMemberById(i).getProperty('email'))
                
            mailman_tool.updateMembers('%s-dev' % self.unix_name,
                                       mails)


    # Cambiamos los miembros
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setMembers')
    def setMembers(self, list, update = True):
        """
        Descripción: Cambiar los miembros.
        Parámetros:  list   = Lista/vector/tupla de los nombres de los miembros.
                     update = Booleano que indica si actualizar el repositorio.
                              Por defecto es True.
        Devuelve:    Nada
        """

        forja_tool = getToolByName(self, 'forja_tool')

        self.members = []
        
        # Eliminamos los miembros que no están ya
        for i in self.__ac_local_roles__.keys():
            if 'Member' in self.__ac_local_roles__[i] and not i.encode() in list and \
               i != self.Creator():
                del self.__ac_local_roles__[i]
                
        # Establecemos los que están (pero no son administradores)
        for i in list:
            if i != self.Creator() and i not in self.administrators:
                self.__ac_local_roles__[i.decode()] = ['Member',]
                self.members = self.members + [i,]

        # Actualizamos el archivo passwd del repositorio
        if update and forja_tool.enable_repository:
            forja_tool.updateProjectRepositoryMembers(self.unix_name,
                                                      self.Creator(),
                                                      self.administrators,
                                                      self.members)

        # Actualizamos la lista de desarrolladores
        if update and forja_tool.enable_maillists:
            mailman_tool = getToolByName(self, 'mailman_tool')
            membership   = getToolByName(self, 'portal_membership')
            mails = []
            for i in self.administrators:
                mails.append(membership.getMemberById(i).getProperty('email'))
                
            for i in self.members:
                mails.append(membership.getMemberById(i).getProperty('email'))
                
            mailman_tool.updateMembers('%s-dev' % self.unix_name,
                                       mails)
        

    # Elimina un usuario del proyecto, independientemente de
    # si es administrador o usuario
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'removeUsers')
    def removeUsers(self, users):
        """
        Descripción: Elimina los usuarios pasados del proyecto.
                     No importa si son administradores o miembros.
        Parámetros:  users = Lista/vector/tupla de los nombres de los miembros para borrar.
        Devuelve:    Nada
        """

        # Los quitamos de administradores
        admins = []
        for i in self.administrators:
            if i not in users:
                admins = admins + [i,]

        self.setAdministrators(admins, False)

        # De usuarios
        members = []
        for i in self.members:
            if i not in users:
                members = members + [i,]

        self.setMembers(members, False)

        # Actualizamos desde aquí el repositorio
        # (así evitamos dos accesos)
        forja_tool = getToolByName(self, 'forja_tool')

        if forja_tool.enable_repository:
            forja_tool.updateProjectRepository(self.unix_name,
                                               self.Creator(),
                                               self.administrators,
                                               self.members)
            
        # Actualizamos la lista de desarrolladores
        if forja_tool.enable_maillists:
            mailman_tool = getToolByName(self, 'mailman_tool')
            membership   = getToolByName(self, 'portal_membership')
            mails = []
            for i in self.administrators:
                mails.append(membership.getMemberById(i).getProperty('email'))
                
            for i in self.members:
                mails.append(membership.getMemberById(i).getProperty('email'))
                
            mailman_tool.updateMembers('%s-dev' % self.unix_name,
                                       mails)


    # Obtener la lista de e-mails de suscriptores a la lista
    security.declareProtected(CMFCorePermissions.View, 'getPublicMailListMembers')
    def getPublicMailListMembers(self):
        """
        Descripción: Obtiene la lista de e-mails de los suscriptores a la lista pública
                     de este proyecto.
        Parámetros:  Ninguno.
        Devuelve:    Una lista con los e-mails de los suscriptores.
        """

        # Herramienta mailman_tool
        mailman_tool = getToolByName(self, 'mailman_tool')

        if mailman_tool is None:
            raise EnvironmentError('mailman_tool not founded')

        # Obtenemos la lista
        return mailman_tool.membersList('%s-users' % self.unix_name)

    # Cambiar la lista de e-mails de suscriptores a la lista
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setPublicMailListMembers')
    def setPublicMailListMembers(self, value):
        """
        Descripción: Actualiza la lista de e-mails de suscriptores de la lista pública.
        Parámetros:  value = La lista con TODOS los suscriptores respecto a la que se
                     actualizará.
        Devuelve:    Nada.
        """

        # Obtenemos la herramienta mailman_tool
        mailman_tool = getToolByName(self, 'mailman_tool')

        if mailman_tool is None:
            raise EnvironmentError('mailman_tool not founded')

        # Actualizamos la lista
        mailman_tool.updateMembers('%s-users' % self.unix_name, value)


    ### Listas de correo ###

    security.declareProtected(CMFCorePermissions.View, 'getPublicMailList')
    def getPublicMailList(self):
        """
        Descripción: Devuelve la dirección de la lista de correo pública.
        Parámetros:  Ninguno.
        Devuelve:    La dirección de la lista de correo.
        """

        mailman_tool = getToolByName(self, 'mailman_tool')
        if mailman_tool is None:
            raise EnvironmentError('mailman_tool not founded')

        return '%s-users@%s' % (self.unix_name.lower(), mailman_tool.domain.lower())

    security.declareProtected(CMFCorePermissions.View, 'getDevMailList')
    def getDevMailList(self):
        """
        Descripción: Devuelve la dirección de la lista de correo de los desarrolladores.
        Parámetros:  Ninguno.
        Devuelve:    La dirección de la lista.
        """

        mailman_tool = getToolByName(self, 'mailman_tool')
        if mailman_tool is None:
            raise EnvironmentError('mailman_tool not founded')

        return '%s-dev@%s' % (self.unix_name.lower(), mailman_tool.domain.lower())

    security.declareProtected(CMFCorePermissions.View, 'addMailToPublicList')
    def addMailToPublicList(self, email):
        """
        Descripción: Añade un e-mail a la lista pública.
        Parámetros:  email = El nuevo e-mail.
        Devuelve:    True si el correo es válido, False si no lo es."""

        mailman_tool = getToolByName(self, 'mailman_tool')

        # Comprobar que es un correo válido
        import re
        expr = re.compile('^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$')
        
        if expr.match(email):
            mailman_tool.addMembers('%s-users' % self.unix_name, (email,))
            return True
        else:
            return False

    security.declareProtected(CMFCorePermissions.View, 'getPublicUnsuscribeMail')
    def getPublicUnsuscribeMail(self):
        """
        Descripción: Devuelve la dirección de correo para eliminar suscripciones de la lista pública.
        Parámetros:  Ninguno.
        Devuelve:    La dirección de correo para darse de baja de la lista.
        """

        mailman_tool = getToolByName(self, 'mailman_tool')

        return '%s-users-unsuscribe@%s' % (self.unix_name.lower(), mailman_tool.domain.lower())

    security.declareProtected(CMFCorePermissions.View, 'getPublicMailListArchive')
    def getPublicMailListArchive(self):
        """
        Descripción: Obtiene el archivo de la lista pública.
        Parámetros:  Ninguno.
        Devuelve:    El código HTML del archivo de la lista.
                     Ajusta los enlaces para que no se rompan.
        """

        mailman_tool = getToolByName(self, 'mailman_tool')

        # ¿Que fichero dentro del archivo?
        path = '/'.join(self.getPhysicalPath()) + '/project_mail_list/'
        fileName = self.REQUEST.PATH_INFO[len(path):] # Eliminamos la parte correspondiente
                                                      # a la URL del objeto.

        # El nombre de archivo tiene dos partes. Obtenemos la de directorio
        # para poder poner correctamente los enlaces
        i = fileName.rfind('/')
        if i != -1:
            fileDir = fileName[:i + 1]
        else:
            fileDir = ''

        # Obtenemos el contenido
        content = mailman_tool.listArchive('%s-users' % self.unix_name, fileName)

        # Cambiamos los enlaces para que funcione
        import re
        p = re.compile('href="(?!(\w+://)|(mailto:))', re.I)
        content = p.sub('href="project_mail_list/%s' % fileDir, content)
        
        # Lo devolvemos
        return content

# Registrar el tipo
registerType(ATProject, PROJECTNAME)
