#
# Utils.py
#
# Bases generales para todos los contenidos
#
# Alejandro Álvarez Ayllón - 29/04/2006
#

from Products.Archetypes.public import BaseContent, BaseFolder
from Products.Archetypes.public import BaseSchema, BaseFolderSchema
from Products.Archetypes.public import BaseBTreeFolder, BaseBTreeFolderSchema
from Products.Archetypes.public import registerType
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from AccessControl import SecurityManager, getSecurityManager

# Funciones útiles

# Schemas

FolderSchema = BaseFolderSchema
FolderSchema['title'].mutator = 'setTitle'
FolderSchema['description'].schemata = 'default'

BTreeFolderSchema = BaseBTreeFolderSchema
BTreeFolderSchema['title'].mutator = 'setTitle'
BTreeFolderSchema['description'].schemata = 'default'

ContentSchema = BaseSchema
ContentSchema['title'].mutator = 'setTitle'
ContentSchema['description'].schemata = 'default'

# Contenido base
class Content:
    """
    Clase Content (abstracta).
    Su única función es establecer el ID del objeto de acuerdo
    a su título.
    """
    def setTitle(self, value):
        """Cambia el ID"""
        plone_tool = getToolByName(self, 'plone_utils')
        self.setId(plone_tool.normalizeString(value))
        self.title = value

 
# Una carpeta que sólo puede contener algunos tipos de archivos
class BaseFilteredFolder(BaseFolder):
    """
    Clase BaseFilteredFolder.
    Limita los tipos que se le puede añadir en función del contenido de
    allowedTypes.
    """

    schema = FolderSchema
 
    allowedTypes = {} # Es una tupla

    # Indica qué tipos se pueden añadir
    def allowedContentTypes(self):
        """Devuelve la lista de tipos admitidos"""
        allowed      = []

        for i in BaseFolder.allowedContentTypes(self):
            if i.getId() in self.allowedTypes:
                allowed.append(i)

        return allowed


# Una carpeta que sólo puede contener algunos tipos de archivos (BTreeFolder)
class BaseBTreeFilteredFolder(BaseBTreeFolder):
    """
    Clase BaseFilteredFolder.
    Limita los tipos que se le puede añadir en función del contenido de
    allowedTypes.
    """

    schema = BTreeFolderSchema
 
    allowedTypes = {} # Es una tupla

    # Indica qué tipos se pueden añadir
    def allowedContentTypes(self):
        """Devuelve la lista de tipos admitidos"""
        allowed      = []

        for i in BaseFolder.allowedContentTypes(self):
            if i.getId() in self.allowedTypes:
                allowed.append(i)

        return allowed
