#
# ATRef_DocFolder.py
#
# Referencias bibliográficas basadas en BibTex
#
# Carpeta contenedora especializada en referencias bibliográficas
#
# Alejandro Álvarez Ayllón - 01/04/2006
#

from Products.Archetypes.public import BaseFolderSchema, Schema, TextField, TextAreaWidget
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import BaseFolder, registerType
from config import PROJECTNAME, REFERENCE_TYPES, REFERENCE_METAS

# Campos
schema = BaseFolderSchema

schema['description'].schemata = 'default'
schema['title'].mutator = 'setTitle' # Método a invocar cuando cambie el campo 'title'

# Clase
class DocumentFolder(BaseFolder):
    """Documentation Folder"""

    schema = schema

    content_icon    = 'document_folder_icon.gif'
    meta_type       = 'ATDocumentFolder'
    portal_type     = 'DocumentFolder'
    archetype_name  = 'Documentation Folder'   
    typeDescription = 'Documentation Folder'

    actions = ({
        'id'         : 'view',
        'name'       : 'View',
        'action'     : 'string:${object_url}/docfolder_view',
        'permissions': (CMFCorePermissions.View,)
        },)

    # Cambia el ID de forma que sea adecuado al título
    def setTitle(self, value):
        """
        Descripción: Cambia el ID del objeto de acuerdo al nuevo título.
        Parámetros:  value = Nuevo título.
        Devuelve:    Nada.
        """
        plone_tool = getToolByName(self, 'plone_utils')
        self.setId(plone_tool.normalizeString(value))
        self.title = value

    #Tipos de referencia
    def getReferenceTypes(self):
        """
        Descripción: Devuelve los tipos de referencias.
        Parámetros:  Ninguno.
        Devuelve:    La lista con los tipos de referencias.
        """
        return REFERENCE_TYPES

    #Tipos admitidos
    def allowedContentTypes(self):
        """
        Descripción: Filtra los tipos de contenido que se pueden agregar.
        Parámetros:  Ninguno.
        Devuelve:    Una lista con los tipos que se pueden añadir.
        """
        allowed = []

        for i in BaseFolder.allowedContentTypes(self):
            if i.getId() in REFERENCE_METAS:
                allowed.append(i)

        return allowed

# Registrar el tipo
registerType(DocumentFolder, PROJECTNAME)
